//+------------------------------------------------------------------+
//|                                                SecureGain_V3.mq4 |
//|                             Copyright 2024, Trade Smart fx Tools |
//|                                 https://tradesmartfxtools.online |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, Trade Smart fx Tools"
#property link      "https://tradesmartfxtools.online"
#property version   "2.00"
#property strict


input int magicNumber = 0607200402; // Magic Number
input double initialsize = 0.01; // Initial size
double lotSize = 0.00;
input int MaxBuy = 10; // Max Buy Orders
input int MaxSell = 10; // Max Sell Orders
input double OverallBuyProfit = 0.2; // OverallBuyProfit (combined for all buy orders).
input double OverallSellProfit = 0.2; // OverallSellProfit (combined for all sell orders).
input int Slippage = 10; // Enter your slippage
input int DistanceBetweenOrders = 50; //  DistanceBetweenOrders
input int NumberofBuy = 3; //  Number of Buy to close deals in 0.01
input int NumberofSell = 3; //  Number of Sell to close deals in 0.01
double profitThreshold = 0.01; // CloseAllOrdersIfProfitIsAbove
double freeMargin = 0.00;
double minLotSize = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MIN);
double maxLotSize = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MAX);


string labelName = "tradesmartfxtools.online";
string labelText = "EA by tradesmartfxtools.online";
string buyProfitLabelName = "OverallBuyProfitLabel";
string sellProfitLabelName = "OverallSellProfitLabel";
int labelFontSize = 18;
color labelColor = Yellow;
color profitLabelColor = Lime;
color lossLabelColor = Red;
int spaceFromBottom = 50;



// Function to check if there are open orders
bool CheckOpenOrders()
  {
   for(int i = 0; i < OrdersTotal(); i++)
     {
      bool order = OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol())
         return(true);
     }
   return(false);
  }


void createOrUpdateLabels(double buyProfit, double sellProfit)
  {
// Create or update the trademark label
   if(ObjectFind(0, labelName) == -1)
     {
      ObjectCreate(0, labelName, OBJ_LABEL, 0, 0, 0);
     }
   ObjectSetInteger(0, labelName, OBJPROP_CORNER, CORNER_LEFT_LOWER);
   ObjectSetInteger(0, labelName, OBJPROP_XDISTANCE, 10);
   ObjectSetInteger(0, labelName, OBJPROP_YDISTANCE, spaceFromBottom);
   ObjectSetInteger(0, labelName, OBJPROP_COLOR, labelColor);
   ObjectSetInteger(0, labelName, OBJPROP_FONTSIZE, labelFontSize);
   ObjectSetInteger(0, labelName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, labelName, OBJPROP_SELECTED, false);
   ObjectSetString(0, labelName, OBJPROP_TEXT, labelText);

// Create or update the buy profit label
   string buyProfitText = "Overall Buy Profit: " + DoubleToString(buyProfit, 2);
   if(ObjectFind(0, buyProfitLabelName) == -1)
     {
      ObjectCreate(0, buyProfitLabelName, OBJ_LABEL, 0, 0, 0);
     }
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_XDISTANCE, 20);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_YDISTANCE, spaceFromBottom - 6); // Adjusted Y position
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_COLOR, profitLabelColor);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_FONTSIZE, labelFontSize);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, buyProfitLabelName, OBJPROP_SELECTED, false);
   ObjectSetString(0, buyProfitLabelName, OBJPROP_TEXT, buyProfitText);

// Create or update the sell profit label
   string sellProfitText = "Overall Sell Profit: " + DoubleToString(sellProfit, 2);
   if(ObjectFind(0, sellProfitLabelName) == -1)
     {
      ObjectCreate(0, sellProfitLabelName, OBJ_LABEL, 0, 0, 0);
     }
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_XDISTANCE, 20);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_YDISTANCE, spaceFromBottom - 36); // Adjusted Y position
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_COLOR, lossLabelColor);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_FONTSIZE, labelFontSize);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(0, sellProfitLabelName, OBJPROP_SELECTED, false);
   ObjectSetString(0, sellProfitLabelName, OBJPROP_TEXT, sellProfitText);
  }


//+------------------------------------------------------------------+
//|  Starting the EA                                                 |
//+------------------------------------------------------------------+
void StartingStep()
{
   if (!CheckOpenOrders())
   {
      lotSize = initialsize;
      
      // Validate the lot size
      string volumeCheckDescription;
      if (!CheckVolumeValue(lotSize, volumeCheckDescription))
      {
         Print(volumeCheckDescription);
         return; // If the lot size is invalid, exit the function
      }

      // Place the sell order
      int SellOrder = OrderSend(_Symbol, OP_SELL, lotSize, Bid, Slippage, 0, 0, "NoLoss_V2 - Starting Sell Order", magicNumber, 0, Red);
      if (SellOrder > 0)
      {
         Print("Sell order opened successfully. Ticket: ", SellOrder);
      }
      else
      {
         Print("Error opening sell order: ", GetLastError());
      }

      // Place the buy order
      int BuyOrder = OrderSend(_Symbol, OP_BUY, lotSize, Ask, Slippage, 0, 0, "NoLoss_V2 - Starting Buy Order", magicNumber, 0, Green);
      if (BuyOrder > 0)
      {
         Print("Buy order opened successfully. Ticket: ", BuyOrder);
      }
      else
      {
         Print("Error opening buy order: ", GetLastError());
      }
   }
}




//+------------------------------------------------------------------+
//|    Close All Buy Orders If Profit Above                          |
//+------------------------------------------------------------------+

void CloseAllBuyOrdersIfProfitAbove(double threshold)
  {
   double totalBuyProfit = 0.0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_BUY)
        {
         totalBuyProfit += OrderProfit();
        }
     }


   if(totalBuyProfit >= threshold)
     {
      for(int i = OrdersTotal() - 1; i >= 0; i--)
        {
         if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_BUY)
           {
            if(OrderClose(OrderTicket(), OrderLots(), Bid, 2, Blue) == true)
              {
               Print("Buy Order closed successfully. Ticket: ", OrderTicket());
              }
            else
              {
               Print("Error closing Buy Order. Error code: ", GetLastError());
              }
           }
        }
     }
  }

//+------------------------------------------------------------------+
//|    Close All Sell Orders If Profit Above                                                              |
//+------------------------------------------------------------------+


void CloseAllSellOrdersIfProfitAbove(double threshold)
  {
   double totalSellProfit = 0.0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_SELL)
        {
         totalSellProfit += OrderProfit();
        }
     }

   if(totalSellProfit >= threshold)
     {
      for(int i = OrdersTotal() - 1; i >= 0; i--)
        {
         if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_SELL)
           {
            if(OrderClose(OrderTicket(), OrderLots(), Ask, 2, Red) == true)
              {
               Print("Sell Order closed successfully. Ticket: ", OrderTicket());
              }
            else
              {
               Print("Error closing Sell Order. Error code: ", GetLastError());
              }
           }
        }
     }
  }


//+------------------------------------------------------------------+
//|    Open Opposite Order If Only One Type                                                              |
//+------------------------------------------------------------------+
void OpenOppositeOrderIfOnlyOneType()
  {
   int totalSellOrders = 0;
   int totalBuyOrders = 0;

   for(int i = 0; i < OrdersTotal(); i++)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol)
        {
         if(OrderType() == OP_SELL)
           {
            totalSellOrders++;
           }
         else
            if(OrderType() == OP_BUY)
              {
               totalBuyOrders++;
              }
        }
     }



   if(totalSellOrders > 0 && totalBuyOrders == 0)
     {


      int orderResult = OrderSend(_Symbol, OP_BUY, initialsize, Ask, Slippage, 0, 0, "NoLoss_V2 - Opposite Buy Order", magicNumber, 0);

      if(orderResult > 0)
        {
         Print("Buy order opened successfully");

        }
      else
        {
         Print("Error opening buy order: ", GetLastError());
        }

     }
   else
      if(totalBuyOrders > 0 && totalSellOrders == 0)
        {

         int orderResult = OrderSend(_Symbol, OP_SELL, initialsize, Bid, Slippage, 0,  0, "NoLoss_V2 - Opposite Sell Order", magicNumber, 0);

         if(orderResult > 0)
           {

            Print("Sell opened successfully");
           }
         else
           {
            Print("Error opening buy order: ", GetLastError());
           }


        }
  }


//+------------------------------------------------------------------+
//|    Calculate Total Last Two Open Buy LotSize                                                              |
//+------------------------------------------------------------------+

double CalculateTotalLastTwoOpenBuyLotSize()
  {
   double totalBuyLotSize = 0.0;
   int buyCount = 0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(buyCount >= 2)
        {
         break;
        }

      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_BUY)
        {
         // Accumulate the lot size for open buy orders
         totalBuyLotSize += OrderLots();
         buyCount++;
        }
     }

   return totalBuyLotSize;
  }

//+------------------------------------------------------------------+
//|   Check And Open Buy Order                                       |
//+------------------------------------------------------------------+

void CheckAndOpenBuyOrder()
{
   string symbol = Symbol();
   int lastBuyOrder = -1;

   for (int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == symbol && OrderType() == OP_BUY)
      {
         lastBuyOrder = i;
         break;
      }
   }

   if (lastBuyOrder >= 0)
   {
      if (OrderSelect(lastBuyOrder, SELECT_BY_POS, MODE_TRADES))
      {
         double openPrice = OrderOpenPrice();
         double currentPrice = SymbolInfoDouble(symbol, SYMBOL_ASK);

         // Check if the symbol has 4 or 5 digits
         int digits = SymbolInfoInteger(symbol, SYMBOL_DIGITS);
         double pipSize = (digits == 5) ? Point * 10 : Point;

         double totalLastTwoOpenBuyLotSize = CalculateTotalLastTwoOpenBuyLotSize();

         // Check the distance between the last buy order and current price
         if ((openPrice - currentPrice) >= DistanceBetweenOrders * pipSize)
         {
            // Validate the lot size before placing the order
            string volumeCheckDescription;
            if (!CheckVolumeValue(totalLastTwoOpenBuyLotSize, volumeCheckDescription))
            {
               Print(volumeCheckDescription);
               return;
            }

            // Place the buy order with the validated lot size
            int orderResult = OrderSend(symbol, OP_BUY, totalLastTwoOpenBuyLotSize, Ask, Slippage, 0, 0, "Buy Order", magicNumber, 0, Blue);

            if (orderResult > 0)
            {
               Print("Buy order opened successfully. Ticket: ", orderResult);
            }
            else
            {
               Print("Error opening buy order: ", GetLastError());
            }
         }
         else
         {
           // Print("Distance between last buy order and current price is less than the required distance. Not opening a new buy order.");
         }
      }
   }
   else
   {
      Print("No open buy orders found.");
   }
}



//+------------------------------------------------------------------+
//|   Calculate Total Last Two Open Sell LotSize                                                            |
//+------------------------------------------------------------------+


double CalculateTotalLastTwoOpenSellLotSize()
  {
   double totalSellLotSize = 0.0;
   int sellCount = 0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(sellCount >= 2)
        {
         break;
        }

      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_SELL)
        {
         totalSellLotSize += OrderLots();
         sellCount++;
        }
     }

   return totalSellLotSize;
  }

//+------------------------------------------------------------------+
//|   Check And Open Sell Order                                      |
//+------------------------------------------------------------------+


void CheckAndOpenSellOrder()
{
   string symbol = Symbol();
   int lastSellOrder = -1;

   for (int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == symbol && OrderType() == OP_SELL)
      {
         lastSellOrder = i;
         break;
      }
   }

   if (lastSellOrder >= 0)
   {
      if (OrderSelect(lastSellOrder, SELECT_BY_POS, MODE_TRADES))
      {
         double openPrice = OrderOpenPrice();
         double currentPrice = SymbolInfoDouble(symbol, SYMBOL_BID);

         // Check if the symbol has 4 or 5 digits
         int digits = SymbolInfoInteger(symbol, SYMBOL_DIGITS);
         double pipSize = (digits == 5) ? Point * 10 : Point;

         double totalLastTwoOpenSellLotSize = CalculateTotalLastTwoOpenSellLotSize();

         // Check the distance between the last sell order and current price
         if ((currentPrice - openPrice) >= DistanceBetweenOrders * pipSize)
         {
            // Validate the lot size before placing the order
            string volumeCheckDescription;
            if (!CheckVolumeValue(totalLastTwoOpenSellLotSize, volumeCheckDescription))
            {
               Print(volumeCheckDescription);
               return;
            }

            // Place the sell order with the validated lot size
            int orderResult = OrderSend(symbol, OP_SELL, totalLastTwoOpenSellLotSize, Bid, Slippage, 0, 0, "Sell Order", magicNumber, 0, Red);

            if (orderResult > 0)
            {
               Print("Sell order opened successfully. Ticket: ", orderResult);
            }
            else
            {
               Print("Error opening sell order: ", GetLastError());
            }
         }
         else
         {
            //Print("Distance between last sell order and current price is less than the required distance. Not opening a new sell order.");
         }
      }
   }
   else
   {
      Print("No open sell orders found.");
   }
}



//+------------------------------------------------------------------+
//|   Get Total Open Sell Orders                                                            |
//+------------------------------------------------------------------+

int GetTotalOpenSellOrders()
  {
   int openSellOrders = 0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_SELL)
        {
         openSellOrders++;
        }
     }

   return openSellOrders;
  }


//+------------------------------------------------------------------+
//|   Get Total Open Buy Orders                                        |
//+------------------------------------------------------------------+

int GetTotalOpenBuyOrders()
  {
   int openBuyOrders = 0;

   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol && OrderType() == OP_BUY)
        {
         openBuyOrders++;
        }
     }

   return openBuyOrders;
  }


//+------------------------------------------------------------------+
//| Check if another order can be placed                             |
//+------------------------------------------------------------------+
bool IsNewOrderAllowed()
  {

   int max_allowed_orders=(int)AccountInfoInteger(ACCOUNT_LIMIT_ORDERS);

   if(max_allowed_orders==0)
      return(true);

   int orders=OrdersTotal();

   return(orders<max_allowed_orders);
  }

////////////////////////////////


//+--------------------------------------------------------------------------------+
//| Close all open orders if total profit is above a specified threshold           |
//+--------------------------------------------------------------------------------+
void CloseOpenTradesIfProfitExceeds(double threshold)
  {
   double totalProfit = 0.0;

// Iterate through all open orders
   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
        {
         // Consider all open trades
         totalProfit += OrderProfit();
        }
     }

// Print("Total Open Trades Profit: ", totalProfit);

// Check if total profit exceeds the threshold
   if(totalProfit > threshold)
     {
      // Close all open orders
      for(int i = OrdersTotal() - 1; i >= 0; i--)
        {
         if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
           {
            // Close the order
            int ticket = OrderTicket();
            double lots = OrderLots();
            double closePrice = OrderType() == OP_BUY ? Bid : Ask;

            if(OrderClose(ticket, lots, closePrice, 10, Red))
              {
               Print(OrderType() == OP_BUY ? "Buy Order" : "Sell Order", " closed successfully. Ticket: ", ticket);
              }
            else
              {
               Print("Error closing Order. Ticket: ", ticket, ", Error code: ", GetLastError());
              }
           }
        }
     }
  }



//+------------------------------------------------------------------+
//|   OnInit Function                                                               |
//+------------------------------------------------------------------+
int OnInit()
  {
//---

   createOrUpdateLabels(0.0, 0.0); // Initialize labels with 0 profit


   StartingStep();


//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---

  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   double totalBuyProfit = 0.0;
   double totalSellProfit = 0.0;

// Calculate total buy and sell profits
   for(int i = OrdersTotal() - 1; i >= 0; i--)
     {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderSymbol() == _Symbol)
        {
         if(OrderType() == OP_BUY)
            totalBuyProfit += OrderProfit();
         else
            if(OrderType() == OP_SELL)
               totalSellProfit += OrderProfit();
        }
     }

// Update all labels with current profit values
   createOrUpdateLabels(totalBuyProfit, totalSellProfit);


   StartingStep();
   OpenOppositeOrderIfOnlyOneType();

   int totalOpenSellOrders = GetTotalOpenSellOrders();

   if(totalOpenSellOrders < MaxSell)
     {
      CheckAndOpenSellOrder();
     }
   else
     {
      Print("Max sell orders reached. Cannot open new sell order.");
     }


////////////////////////////////////////////////////



   int totalOpenBuyOrders = GetTotalOpenBuyOrders();

   if(totalOpenBuyOrders < MaxBuy)
     {
      CheckAndOpenBuyOrder();
     }
   else
     {
      Print("Max buy orders reached. Cannot open new buy order.");
     }






/////////////////////////////////////////

   if(totalOpenBuyOrders >= NumberofBuy)
     {
      CloseAllBuyOrdersIfProfitAbove(0.01);
     }
   else
     {
      CloseAllBuyOrdersIfProfitAbove(OverallBuyProfit);
     }

//------------------------

   if(totalOpenSellOrders >= NumberofSell)
     {
      CloseAllSellOrdersIfProfitAbove(0.01);
     }
   else
     {
      CloseAllSellOrdersIfProfitAbove(OverallSellProfit);
     }

/////////////////////////////////////////


   CloseOpenTradesIfProfitExceeds(profitThreshold);



  }
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Check the correctness of the order volume                        |
//+------------------------------------------------------------------+
bool CheckVolumeValue(double volume, string &description)
{
   //--- minimal allowed volume for trade operations
   double min_volume = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_MIN);
   if (volume < min_volume)
   {
      description = StringFormat("Volume is less than the minimal allowed SYMBOL_VOLUME_MIN=%.2f", min_volume);
      return (false);
   }

   //--- maximal allowed volume of trade operations
   double max_volume = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_MAX);
   if (volume > max_volume)
   {
      description = StringFormat("Volume is greater than the maximal allowed SYMBOL_VOLUME_MAX=%.2f", max_volume);
      return (false);
   }

   //--- get minimal step of volume changing
   double volume_step = SymbolInfoDouble(Symbol(), SYMBOL_VOLUME_STEP);

   int ratio = (int)MathRound(volume / volume_step);
   if (MathAbs(ratio * volume_step - volume) > 0.0000001)
   {
      description = StringFormat("Volume is not a multiple of the minimal step SYMBOL_VOLUME_STEP=%.2f, the closest correct volume is %.2f",
                                 volume_step, ratio * volume_step);
      return (false);
   }
   
   description = "Correct volume value";
   return (true);
}
